# -*- coding: utf-8 -*-
''' bugle
a generator of static sites
'''
import errno
import os


class Bugle(object):
    ''' finding source files (entries)
    validating these files
    compiles tags
    generates slugs
    creates indices for each tag
    injects rendered data into templates
    sets up css and js
    saves the lot in an appropriate folder structure
    '''

    def __init__(self, source_path, out_path):
        self.source_path = source_path
        self.entry_path = os.path.join(source_path, 'entries')
        self.template_path = os.path.join(source_path, 'templates')
        self.meta_path = os.path.join(source_path, 'meta')
        self.css_path = os.path.join(source_path, 'static/css')
        self.js_path = os.path.join(source_path, 'static/js')
        self.out_path = out_path


    def discover_files(self, directory):
        ''' recursively dig into a path, looking for entries
        this discovery allows authors to organize files however they'd like

        returns set('/notes/hola.md', 'bike.md')
        '''
        filepaths = []
        for root, subdirectories, filenames in os.walk(directory):
            for filename in filenames:
                filepaths.append(os.path.join(root, filename))

            for subdirectory in subdirectories:
                self.discover_files(subdirectory)

        return set(filepaths)


    def compile_tags(self, entries):
        ''' find all unique tags among the specified entries
        counts entries belonging to that tag
        sorts based on the count

        returns [{'tag': 'python', 'count': 8}, {'tag': 'twilio', 'count': 4}]
        '''
        tags = []
        for entry in entries:
            tags.extend(entry.config['tags'])

        # de-duplicate
        tags = list(set(tags))

        # count entries associated with each tag
        counted_tags = []
        for tag in tags:
            count = 0
            for entry in entries:
                if tag in entry.config['tags']:
                    count += 1
            counted_tags.append({'name': tag, 'count': count})

        # sort by count attr
        counted_tags.sort(key=lambda t: t['count'], reverse=True)

        return counted_tags


    def verify_unique_routes(self, entries, meta_files):
        ''' routes generated from entries, tags and meta files should be unique
        '''
        tags = self.compile_tags(entries)
        tag_slugs = [tag['name'].replace(' ', '-') for tag in tags]

        entry_slugs = [e.slug for e in entries]
        meta_slugs = [m.slug for m in meta_files]

        all_slugs = []
        all_slugs.extend(tag_slugs)
        all_slugs.extend(entry_slugs)
        all_slugs.extend(meta_slugs)

        # creating a set will eliminate duplicates
        if len(all_slugs) != len(set(all_slugs)):
            return False
        else:
            return True

    def ensure_path_exists(self, path):
        # from http://stackoverflow.com/a/5032238/232638
        try:
            os.makedirs(path)
        except OSError as exception:
            if exception.errno != errno.EEXIST:
                raise



# -*- coding: utf-8 -*-
''' meta
one of the meta docs
'''
import re

import markdown
from unidecode import unidecode
import yaml


class Meta(object):
    ''' stores raw dta
    validates
    generates a slug
    '''

    def __init__(self, filepath):
        # the separator between yaml and markdown
        self.separator = '---'

        self.filepath = filepath

        with open(filepath, 'r') as f:
            self.raw = f.read()

        if self.validate():
            # save the config and markdown
            self.parse()
            # save a slug based on title or route param
            self.generate_slug()


    def __repr__(self):
        return 'Meta %s' % self.filepath


    def validate(self):
        ''' checks that things are properly formatted
        needs one separator in the file
        content before the separator needs to be yaml

        returns (True|False, 'rationale')
        '''
        data = self.raw.split(self.separator)
        # validate the presence of a separator
        if len(data) < 2:
            self.validation_message = ('the separator "%s" needs to appear at'
                    ' least once' % self.separator)
            self.valid = False
            return False

        # validate the yaml
        try:
            config = yaml.load(data[0])
        except:
            self.validation_message = 'invalid yaml header'
            self.valid = False
            return False

        # validates that all required params are present and not empty
        required_params = ['title']
        for param in required_params:
            if param not in config.keys():
                self.validation_message = ('yaml header is missing parameter'
                    ' "%s"' % param)
                self.valid = False
                return False

            if not config[param]:
                self.validation_message = ('yaml parameter "%s" is invalid' 
                        % param)
                self.valid = False
                return False


        self.validation_message = 'valid.'
        self.valid = True
        return True


    def parse(self):
        ''' saves config and markdown
        creates a 'last_updated' param based on 'created' and 'updated'

        returns None
        '''
        data = self.raw.split(self.separator)
        self.config = yaml.load(data.pop(0))
        # in case the separator was in the markdown, use it in the rejoin
        self.markdown = self.separator.join(data)
        self.rendered_markdown = markdown.markdown(self.markdown)


    def generate_slug(self):
        ''' generate a slug
        technique from http://flask.pocoo.org/snippets/5/

        returns None
        '''
        if 'route' in self.config.keys():
            route = self.config['route']
        else:
            route = self.config['title']

        if route == '':
            self.slug = ''
            return

        punctuation = re.compile(r'[\t !"#$%&\'()*\-/<=>?@\[\\\]^_`{|},.]+')
        result = []
        for word in punctuation.split(route.lower()):
            result.extend(unidecode(word).split())

        self.slug = unicode(u'-'.join(result))



# -*- coding: utf-8 -*-
"""
bugle
~~~~

yet another static site-generator

"""

from setuptools import setup

with open('requirements.txt') as f:
    libs = f.readlines()

packages = [
        'bugle'
]

setup(
    name='bugle-sites',
    version='0.0.4',
    url='https://github.com/yosemitebandit/bugle',
    license='MIT',
    author='Matt Ball',
    author_email='matt.ball.2@gmail.com',
    description='static sites from markdown and yaml',
    long_description=__doc__,
    packages=packages,
    zip_safe=False,
    include_package_data=True,
    install_requires=libs,
    platforms='any',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
